/**********************************************************************************************
An M/M/S/K System Example
Written by: Minh Viet Le - Monash University - January 20, 2006
Email: minh.leviet@infotech.monash.edu.au

Problem Statement

A government office has 4 officers. The room is capable of having maximum 7 customers including 
customers being served. Customers arrive at a mean rate of 15 people per hour (lamda=12). If there is no
more room customers must go elsewhere. The time required to serve each customer with a mean of 
12 min (muy=60/12=5ppl/hr). 

Determine 
(a) What is the average number of customers in the office?
(b) The expected rate at which customers are denied entrance into the office.
(c) The probability that there are more than three customers arriving at the office at the same time?
(d) The probability that a customer will spend more than t minutes (e.g. 3 mins) in the office.
(e) The average time a customer spend in the office. In other words, how much time should 
a customer expect to spend in the office?
(f) The average number of customers waiting to be served.

Descriptions/assumptions of the solution
 
Let lamda be the average customer arrival in per unit time (e.g. hour, minute, second).
Let muy be the average service rate of customers (i.e. the number of customers served per unit time).

Therefore, the expected interarrival time and the expected time to serve one customer 
are 1/lamda and 1/muy, respectively.

Let pro be the traffic intensity (or the ultilisation factor) 
			
			pro=lamda/muy 
		or  pro=lamda/s*muy (s is the number of servers in the system)

Let L  be the average number of customers in the system.
Let Lq be the average length of the queue (i.e. the average number of customers waiting to be served).
Let W  be the average time a customer is expected to spend in the system.
Let Wq be the average time a customer is expected to spend in the queue.

Let PSt be the probability that a customer spends more than t units of time in the system (bank).
Let PQt be the probability that a customer spends more than t units of time in the queue.

Let s be the number of servers in the system.
Let Pn be the probability of the system in state n (e.g. with 2 servers, P3 is the 
probability that one customer is in the queue and 2 customers are being served.

	Pn= ([(s*pro)^n]/n!) * P0			(n=0,1,2,...,s)			in M/M/S systems
or
	Pn= ([(s^s) * pro]/s!) * P0			(n=s+1,s+2,...)			in M/M/S systems

			
	P0= [(s^s) * pro^(s+1)] / [(s!) * (1-pro)] +  
		+ Sigma(n=0 -> s)([(s * pro)^n]/ n!)]^-1				in M/M/S systems


    
	P0= [([s^s * pro^(s+1) * (1-pro^(K-s))]/[s!*(1-pro)]) +			(pro # 1) in M/M/S/K systems
						+ Sigma(n=0->s)([(s * pro)^n]/ n!)]^-1

  or

    P0= [([s^s * (K-s)]/s!) 
						+ Sigma(n=0->s)([(s * pro)^n]/ n!)]^-1		(pro = 1) in M/M/S/K systems


	Pn= ([(s * pro)^n]/n!) * P0		(n=1,2,...,s)						in M/M/S/K systems

  or

	Pn= [(s^s * pro^n) / s!] * P0	(n=s+1,...,K)						in M/M/S/K systems

	Pn=0							(n=K+1,K+2,...)						in M/M/S/K systems


Let lamda_sys be the average arrival rate of customers into the system (generally, equally to the queue).
In general, 
		
	lamda_sys = lamda 
	lamda_sys = lamda(1-Pk)  in M/M/S/K systems

The Little's formulas

	L=lamda_sys * W				
	Lq=lamda_sys * Wq

	Lq= ([s^s * pro^(s+1)] / ([s! * (1-pro)^2])/([(1-pro^(K-s)-(1-pro)(K-s)(pro^(K-s)] * P0) in M/M/S/K systems

Assuming all the assumptions are met.

**********************************************************************************************/

#include<stdio.h>
#include<math.h>
#include<stdlib.h>

int fact(int n); /*factorial function*/
float PN(unsigned int n, unsigned int s, unsigned int k, float pro, float P0);	/*calculate probability of the system*/
														/*in state n*/
int main(int argc, char* argv[])
{
	float lamda=0, lamda_sys=0, muy=0, pro=0;
	float L=0, Lq=0, W=0, Wq=0, t=0.05;			//default t=3 minutes = 0.05 hrs

	float PQt=0, PSt=0, temp=0, temp2=0, temp3=0, temp4=0, temp5=0, temp6=0;

	float P0, P0a, P1, P2, P3, P4, Pk, Wt=0;
	
	unsigned int s, n, k;

	printf("An M/M/S/K System Example.\n");
	printf("Assuming time units in hour.\n");
	
	printf("\n");

	printf("Enter the average customer arrival in one hour: ");
	scanf("%f",&lamda);

	printf("Enter the average service rate of customers in one hour e.g. 6 customers/1hr: ");
	scanf("%f",&muy);

	printf("Enter the number of servers: ");
	scanf("%d",&s);

	printf("Enter the maximum capacity of the room (i.e. number of customers being served and waiting): ");
	scanf("%d",&k);

	printf("Enter the time that the customer is probably required to spend at the office\n"); 
	printf("(default t=3 minutes = 0.05 hrs) to find out its probability :");
	scanf("%f",&t);

	printf("t is %f",t);

	printf("\n");

	pro = (float)(lamda/(s*muy));			//s is the number of servers

	if (pro == 1)
	{
		printf("The value of pro is %.5f\n",pro);
		
		printf("\n");
		
		P0a=0;
		for(n=0;n<=s;n++)
			P0a += (float) pow((s * pro),n)/fact(n);

		P0=1/((((float)pow(s,s) * (k-s))/fact(s)) + P0a);
	}
	else
	{
		printf("The value of pro is %.5f\n",pro);
		
		printf("\n");
		
		P0a=0;
		for(n=0;n<=s;n++)
			P0a += (float) pow((s * pro),n)/fact(n);

		P0=1/((((float)pow(s,s) * (float)pow(pro,s+1) * (1-(float)pow(pro,k-s)))/(fact(s) * (1-pro))) + P0a);
	}
	
		P1=PN(1,s,k,pro,P0);
		P2=PN(2,s,k,pro,P0);
		P3=PN(3,s,k,pro,P0);
		P4=PN(4,s,k,pro,P0);
		Pk=PN(k,s,k,pro,P0);

		printf("The value of P0 is %.5f\n",P0);
		
		printf("The value of P1 is %.5f\n",P1);

		printf("The value of P2 is %.5f\n",P2);

		printf("The value of P3 is %.5f\n",P3);

		printf("The value of P4 is %.5f\n",P4);
		
		printf("The value of P%d is %.5f\n",k,Pk);

		printf("\n");

		/**********************************************************************************/
		/*Wt is propability of the time spent in the system of a customer 
		(in this case the probability of a customer is expected to spend in the office 
		(queuing time + serving time)*/
		
		temp4 = (float)pow((s * pro),s) * P0;

		temp5= muy * t * (s - 1 - s * pro);
		
		temp3 = temp4 * (1 - (float)exp(-temp5));

		temp6=(fact(s) * (1-pro) * (s - 1 - s*pro));

		if (temp6==0)
			temp2=0;
		else
			temp2=temp3/temp6;

		temp=(1 + temp2);
		
		Wt= (float)exp(-muy * t) * temp;
		/***********************************************************************************/		
		
		Lq=(((float)pow(s,s) * (float)pow(pro,s+1) * P0)/(fact(s) * (float)pow(1-pro,2))) * (1 - (float)pow(pro,k-s) - (1-pro) * (k-s) * (float)pow(pro,k-s)) ;	//The average number of calls in the queue.

		lamda_sys = lamda * (1-Pk);
			
		Wq=Lq/lamda_sys;				//The average time a customer spend in the queue.

		W=Wq + 1/muy;					//The average time a customer (call) expects to spend in the office.
										//It holds whenever a single expected service time, 1/muy for all customers (calls)
		
		L= W * lamda_sys;				//The average number of customers in the office.

		
		printf("The average rate of customers arriving into the office is %.3f (people/hrs)\n",lamda_sys);
		
		printf("\n");

		printf("The average number of customers in the office is %.3f (people)\n",L);
		
		printf("\n");

		printf("The expected rate at which customers are denied entrance into the office is %.5f people/hr\n",lamda-lamda_sys);
		
		printf("\n");

		printf("The probability that there are more than three customers \n"); 
		printf("arriving at the office at the same time is %.5f \n", 1-(P0+P1+P2+P3));
		
		printf("\n");

		printf("The probability that a customer is required to spend more than \n");
		printf("%.5f hours in the office is %.5f \n",t,Wt);
		
		printf("\n");
		
		printf("The average time a customer is expected to spend in the office is %.5f (hours)\n",W);
		
		printf("\n");
		
		printf("The average number of customers waiting to be served (the average queue length) is %.5f (people)\n",Lq);

		printf("\n");				
	
	return 0;
}

int fact(int n)
{
	if (n<0)
		exit(0);
	else if((n==1) || (n==0)) 
		return 1;
	else
		return n*fact(n-1);
}

float PN(unsigned int n, unsigned int s, unsigned int k, float pro, float P0)
{
	if((n>0) && (n<=s))
		return (float)(pow((s*pro),n)*P0)/fact(n);
	
	else if((s < n) && (n <= k))
		return (float)(pow(s,s) * pow(pro,n) * P0)/fact(s);
	
	else
		return 0;

}
